@@ -329,6 +329,22 @@ class TestWaitersObjects(unittest.TestCase):
         waiter.wait()
         self.assertEqual(operation_method.call_count, 3)
 
+    def test_waiter_matches_with_invalid_error_response(self):
+        # Verify that the call will not raise WaiterError
+        # because of 'Error' key in success response.
+        config = self.create_waiter_config(
+            max_attempts=3,
+            acceptors=[{'state': 'success', 'matcher': 'path',
+                        'argument': 'Foo', 'expected': 'SUCCESS'}])
+        operation_method = mock.Mock()
+        waiter = Waiter('MyWaiter', config, operation_method)
+        self.client_responses_are(
+            {'Foo': 'SUCCESS', 'Error': 'foo'},
+            for_operation=operation_method
+        )
+        waiter.wait()
+        self.assertEqual(operation_method.call_count, 1)
+
     def test_waiter_never_matches(self):
         # Verify that a matcher will fail after max_attempts
         # is exceeded.
@@ -361,6 +377,17 @@ class TestWaitersObjects(unittest.TestCase):
         with self.assertRaises(WaiterError):
             waiter.wait()
 
+    def test_last_response_available_on_waiter_error(self):
+        last_response = {'Error': {'Code': 'UnknownError', 'Message': 'bad error'}}
+        config = self.create_waiter_config()
+        operation_method = mock.Mock()
+        self.client_responses_are(last_response,
+                                  for_operation=operation_method)
+        waiter = Waiter('MyWaiter', config, operation_method)
+        with self.assertRaises(WaiterError) as e:
+            waiter.wait()
+        self.assertEqual(e.exception.last_response, last_response)
+
     def test_unspecified_errors_propagate_error_code(self):
         # If a waiter receives an error response, then the
         # waiter should pass along the error code
@@ -381,6 +408,56 @@ class TestWaitersObjects(unittest.TestCase):
         with self.assertRaisesRegexp(WaiterError, error_message):
             waiter.wait()
 
+    def _assert_failure_state_error_raised(self, acceptors, responses, expected_msg):
+        config = self.create_waiter_config(
+            acceptors=acceptors)
+        operation_method = mock.Mock()
+        waiter = Waiter('MyWaiter', config, operation_method)
+        self.client_responses_are(
+            *responses,
+            for_operation=operation_method
+        )
+        with self.assertRaisesRegexp(WaiterError, expected_msg):
+            waiter.wait()
+
+    def test_waiter_failure_state_error(self):
+        test_cases = [
+            ([{'state': 'failure', 'matcher': 'path',
+               'argument': 'Foo', 'expected': 'FAILURE'}],
+             [{'Foo': 'FAILURE'}],
+             'FAILURE'),
+            ([{'state': 'failure', 'matcher': 'pathAll',
+               'argument': 'Tables[].State', 'expected': 'FAILURE'}],
+             [{'Tables': [{"State": "FAILURE"}]}],
+             'FAILURE'),
+            ([{'state': 'failure', 'matcher': 'pathAny',
+               'argument': 'Tables[].State', 'expected': 'FAILURE'}],
+             [{'Tables': [{"State": "FAILURE"}]}],
+             'FAILURE'),
+            ([{'state': 'failure', 'matcher': 'status', 'expected': 404}],
+             [{'ResponseMetadata': {'HTTPStatusCode': 404}}],
+             '404'),
+            ([{'state': 'failure', 'matcher': 'error', 'expected': 'FailError'}],
+             [{'Error': {'Code': 'FailError', 'Message': 'foo'}}],
+             'FailError'),
+            ([{'state': 'retry', 'matcher': 'error', 'expected': 'RetryMe'}],
+             [{'Success': False}]*4,
+             'Max attempts exceeded'),
+            ([
+                {'state': 'success', 'matcher': 'status', 'expected': 200},
+                {'state': 'retry', 'matcher': 'error', 'expected': 'RetryMe'},
+            ],
+             [{'Success': False},
+              {'Error': {'Code': 'RetryMe', 'Message': 'foo'}},
+              {'Success': False},
+              {'Success': False},
+              ],
+             'Previously accepted state'),
+        ]
+
+        for acceptors, responses, expected_msg in test_cases:
+            self._assert_failure_state_error_raised(acceptors, responses, expected_msg)
+
     def test_waiter_transitions_to_failure_state(self):
         acceptors = [
             # A success state that will never be hit.
@@ -425,24 +502,6 @@ class TestWaitersObjects(unittest.TestCase):
         waiter.wait()
         self.assertEqual(operation_method.call_count, 3)
 
-    def test_waiter_transitions_to_retry_but_max_attempts_exceeded(self):
-        acceptors = [
-            {'state': 'success', 'matcher': 'status', 'expected': 200},
-            {'state': 'retry', 'matcher': 'error', 'expected': 'RetryMe'},
-        ]
-        config = self.create_waiter_config(acceptors=acceptors)
-        operation_method = mock.Mock()
-        self.client_responses_are(
-            {'Success': False},
-            {'Error': {'Code': 'RetryMe', 'Message': 'foo'}},
-            {'Success': False},
-            {'Success': False},
-            for_operation=operation_method
-        )
-        waiter = Waiter('MyWaiter', config, operation_method)
-        with self.assertRaises(WaiterError):
-            waiter.wait()
-
     def test_kwargs_are_passed_through(self):
         acceptors = [
             {'state': 'success', 'matcher': 'error', 'expected': 'MyError'},
@@ -482,6 +541,41 @@ class TestWaitersObjects(unittest.TestCase):
         self.assertEqual(sleep_mock.call_count, 2)
         sleep_mock.assert_called_with(delay_time)
 
+    @mock.patch('time.sleep')
+    def test_waiter_invocation_config_honors_delay(self, sleep_mock):
+        config = self.create_waiter_config()
+        operation_method = mock.Mock()
+        self.client_responses_are(
+            {'Success': False},
+            {'Success': False},
+            {'Success': False},
+            for_operation=operation_method
+        )
+        waiter = Waiter('MyWaiter', config, operation_method)
+        custom_delay = 3
+        with self.assertRaises(WaiterError):
+            waiter.wait(WaiterConfig={'Delay': custom_delay})
+
+        # We attempt three times, which means we need to sleep
+        # twice, once before each subsequent request.
+        self.assertEqual(sleep_mock.call_count, 2)
+        sleep_mock.assert_called_with(custom_delay)
+
+    def test_waiter_invocation_config_honors_max_attempts(self):
+        config = self.create_waiter_config()
+        operation_method = mock.Mock()
+        self.client_responses_are(
+            {'Success': False},
+            {'Success': False},
+            for_operation=operation_method
+        )
+        waiter = Waiter('MyWaiter', config, operation_method)
+        custom_max = 2
+        with self.assertRaises(WaiterError):
+            waiter.wait(WaiterConfig={'MaxAttempts': custom_max})
+
+        self.assertEqual(operation_method.call_count, 2)
+
 
 class TestCreateWaiter(unittest.TestCase):
     def setUp(self):
